"""Synology DSM tests."""

# pylint: disable=protected-access
import pytest
from aiohttp import ClientTimeout

from synology_dsm.api.core.external_usb import SynoCoreExternalUSB
from synology_dsm.api.core.security import SynoCoreSecurity
from synology_dsm.api.core.share import SynoCoreShare
from synology_dsm.api.core.system import SynoCoreSystem
from synology_dsm.api.core.upgrade import SynoCoreUpgrade
from synology_dsm.api.core.utilization import SynoCoreUtilization
from synology_dsm.api.download_station import SynoDownloadStation
from synology_dsm.api.dsm.information import SynoDSMInformation
from synology_dsm.api.photos import SynoPhotos
from synology_dsm.api.storage.storage import SynoStorage
from synology_dsm.api.surveillance_station import SynoSurveillanceStation
from synology_dsm.const import API_AUTH, API_INFO
from synology_dsm.exceptions import (
    SynologyDSMAPIErrorException,
    SynologyDSMAPINotExistsException,
    SynologyDSMLogin2SAFailedException,
    SynologyDSMLogin2SARequiredException,
    SynologyDSMLoginFailedException,
    SynologyDSMLoginInvalidException,
    SynologyDSMNotLoggedInException,
    SynologyDSMRequestException,
)

from . import (
    USER_MAX_TRY,
    VALID_HOST,
    VALID_HTTPS,
    VALID_PASSWORD,
    VALID_PORT,
    VALID_USER,
    VALID_USER_2SA,
    SynologyDSMMock,
)


class TestSynologyDSM:
    """Common SynologyDSM 5 and 6 test cases."""

    def test_init(self, dsm):
        """Test init."""
        assert dsm.username == VALID_USER
        assert str(dsm._base_url) == f"https://{VALID_HOST}:{VALID_PORT}"
        assert dsm._aiohttp_timeout.total == 10
        assert not dsm.apis.get(API_AUTH)
        assert not dsm._session_id

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.asyncio
    async def test_login_neccessary(self, version):
        """Test login is neccessary."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            USER_MAX_TRY,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version

        with pytest.raises(SynologyDSMNotLoggedInException) as error:
            await dsm.utilisation.update()
        error_value = error.value.args[0]
        assert error_value["api"] is None
        assert error_value["code"] == -1
        assert error_value["reason"] == "Unknown"
        assert error_value["details"] == "Not logged in. You have to do login() first."

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.asyncio
    async def test_login_basic_failed(self, version):
        """Test basic failed login."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            USER_MAX_TRY,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version

        with pytest.raises(SynologyDSMLoginFailedException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 407
        assert error_value["reason"] == "Max Tries (if auto blocking is set to true)"
        assert error_value["details"] == USER_MAX_TRY

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.asyncio
    async def test_login_2sa_failed(self, version):
        """Test failed login with 2SA."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER_2SA,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version

        with pytest.raises(SynologyDSMLogin2SARequiredException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 403
        assert error_value["reason"] == "One time password not specified"
        assert (
            error_value["details"]
            == "Two-step authentication required for account: valid_user_2sa"
        )

        with pytest.raises(SynologyDSMLogin2SAFailedException) as error:
            await dsm.login(888888)
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 404
        assert error_value["reason"] == "One time password authenticate failed"
        assert (
            error_value["details"]
            == "Two-step authentication failed, retry with a new pass code"
        )

        assert dsm._session_id is None
        assert dsm._syno_token is None
        assert dsm._device_token is None

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.asyncio
    async def test_connection_failed(self, version):
        """Test failed connection."""
        # No internet
        dsm = SynologyDSMMock(
            None,
            "no_internet",
            VALID_PORT,
            VALID_USER,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMRequestException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert not error_value["api"]
        assert error_value["code"] == -1
        assert error_value["reason"] == "Unknown"
        assert (
            "ClientError = <urllib3.connection.VerifiedHTTPSConnection "
            in error_value["details"]
        )

        assert not dsm.apis.get(API_AUTH)
        assert not dsm._session_id

        # Wrong host
        dsm = SynologyDSMMock(
            None,
            "host",
            VALID_PORT,
            VALID_USER,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMRequestException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert not error_value["api"]
        assert error_value["code"] == -1
        assert error_value["reason"] == "Unknown"
        assert (
            "ClientError = <urllib3.connection.HTTPConnection "
            in error_value["details"]
        )

        assert not dsm.apis.get(API_AUTH)
        assert not dsm._session_id

        # Wrong port
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            0,
            VALID_USER,
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMRequestException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert not error_value["api"]
        assert error_value["code"] == -1
        assert error_value["reason"] == "Unknown"
        assert error_value["details"] == (
            "ClientError = [SSL: WRONG_VERSION_NUMBER] "
            "wrong version number (_ssl.c:1076)"
        )

        assert not dsm.apis.get(API_AUTH)
        assert not dsm._session_id

        # Wrong HTTPS
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER,
            VALID_PASSWORD,
            False,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMRequestException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert not error_value["api"]
        assert error_value["code"] == -1
        assert error_value["reason"] == "Unknown"
        assert error_value["details"] == "ClientError = Bad request"

        assert not dsm.apis.get(API_AUTH)
        assert not dsm._session_id

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.asyncio
    async def test_login_failed(self, version):
        """Test failed login."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            "user",
            VALID_PASSWORD,
            VALID_HTTPS,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMLoginInvalidException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 400
        assert error_value["reason"] == "Invalid credentials"
        assert error_value["details"] == "Invalid password or not admin account: user"

        assert dsm.apis.get(API_AUTH)
        assert not dsm._session_id

        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER,
            "pass",
            VALID_HTTPS,
        )
        dsm.dsm_version = version
        with pytest.raises(SynologyDSMLoginInvalidException) as error:
            await dsm.login()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.API.Auth"
        assert error_value["code"] == 400
        assert error_value["reason"] == "Invalid credentials"
        assert (
            error_value["details"]
            == "Invalid password or not admin account: valid_user"
        )

        assert dsm.apis.get(API_AUTH)
        assert not dsm._session_id

    @pytest.mark.parametrize("version", [5, 6, 7])
    @pytest.mark.parametrize(
        "timeout,expected_result",
        [
            (2, (2, None)),
            (15, (15, None)),
            (ClientTimeout(total=5), (5, None)),
            (ClientTimeout(total=60, connect=15), (60, 15)),
        ],
    )
    def test_request_timeout(self, version, timeout, expected_result):
        """Test request timeout."""
        dsm = SynologyDSMMock(
            None,
            VALID_HOST,
            VALID_PORT,
            VALID_USER,
            VALID_PASSWORD,
            VALID_HTTPS,
            timeout=timeout,
        )
        dsm.dsm_version = version
        assert dsm._aiohttp_timeout.total == expected_result[0]
        assert dsm._aiohttp_timeout.connect == expected_result[1]

    @pytest.mark.asyncio
    async def test_request_get(self, dsm):
        """Test get request."""
        await dsm.login()
        assert await dsm.get(API_INFO, "query")
        assert await dsm.get(API_AUTH, "login")
        assert await dsm.get("SYNO.DownloadStation2.Task", "list")
        assert await dsm.get(API_AUTH, "logout")

    @pytest.mark.asyncio
    async def test_request_get_failed(self, dsm):
        """Test failed get request."""
        await dsm.login()
        with pytest.raises(SynologyDSMAPINotExistsException) as error:
            await dsm.get("SYNO.Virtualization.API.Task.Info", "list")
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.Virtualization.API.Task.Info"
        assert error_value["code"] == -2
        assert error_value["reason"] == "Unknown"
        assert (
            error_value["details"]
            == "API SYNO.Virtualization.API.Task.Info does not exists"
        )

    @pytest.mark.asyncio
    async def test_request_post(self, dsm):
        """Test post request."""
        await dsm.login()
        assert await dsm.post(
            "SYNO.FileStation.Upload",
            "upload",
            params={"dest_folder_path": "/upload/test", "create_parents": True},
            files={"file": "open('file.txt','rb')"},
        )

        assert await dsm.post(
            "SYNO.DownloadStation2.Task",
            "create",
            params={
                "uri": "ftps://192.0.0.1:21/test/test.zip",
                "username": "admin",
                "password": "1234",
            },
        )

    @pytest.mark.asyncio
    async def test_request_post_failed(self, dsm):
        """Test failed post request."""
        await dsm.login()
        with pytest.raises(SynologyDSMAPIErrorException) as error:
            await dsm.post(
                "SYNO.FileStation.Upload",
                "upload",
                params={"dest_folder_path": "/upload/test", "create_parents": True},
                files={"file": "open('file_already_exists.txt','rb')"},
            )
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.FileStation.Upload"
        assert error_value["code"] == 1805
        assert error_value["reason"] == (
            "Can’t overwrite or skip the existed file, if no overwrite"
            " parameter is given"
        )
        assert not error_value["details"]

        with pytest.raises(SynologyDSMAPIErrorException) as error:
            await dsm.post(
                "SYNO.DownloadStation2.Task",
                "create",
                params={
                    "uri": "ftps://192.0.0.1:21/test/test_not_exists.zip",
                    "username": "admin",
                    "password": "1234",
                },
            )
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.DownloadStation2.Task"
        assert error_value["code"] == 408
        assert error_value["reason"] == "File does not exist"
        assert not error_value["details"]

    def test_reset_str_attr(self, dsm):
        """Test reset with string attr."""
        assert not dsm._external_usb
        assert dsm.external_usb
        assert dsm._external_usb
        assert dsm.reset("external_usb")
        assert not dsm._external_usb

        assert not dsm._security
        assert dsm.security
        assert dsm._security
        assert dsm.reset("security")
        assert not dsm._security

        assert not dsm._share
        assert dsm.share
        assert dsm._share
        assert dsm.reset("share")
        assert not dsm._share

        assert not dsm._system
        assert dsm.system
        assert dsm._system
        assert dsm.reset("system")
        assert not dsm._system

        assert not dsm._upgrade
        assert dsm.upgrade
        assert dsm._upgrade
        assert dsm.reset("upgrade")
        assert not dsm._upgrade

        assert not dsm._utilisation
        assert dsm.utilisation
        assert dsm._utilisation
        assert dsm.reset("utilisation")
        assert not dsm._utilisation

        assert not dsm._download
        assert dsm.download_station
        assert dsm._download
        assert dsm.reset("download")
        assert not dsm._download

        assert not dsm._photos
        assert dsm.photos
        assert dsm._photos
        assert dsm.reset("photos")
        assert not dsm._photos

        assert not dsm._storage
        assert dsm.storage
        assert dsm._storage
        assert dsm.reset("storage")
        assert not dsm._storage

        assert not dsm._surveillance
        assert dsm.surveillance_station
        assert dsm._surveillance
        assert dsm.reset("surveillance")
        assert not dsm._surveillance

    def test_reset_str_key(self, dsm):
        """Test reset with string API key."""
        assert not dsm._external_usb
        assert dsm.external_usb
        assert dsm._external_usb
        assert dsm.reset(SynoCoreExternalUSB.API_KEY)
        assert not dsm._external_usb

        assert not dsm._security
        assert dsm.security
        assert dsm._security
        assert dsm.reset(SynoCoreSecurity.API_KEY)
        assert not dsm._security

        assert not dsm._share
        assert dsm.share
        assert dsm._share
        assert dsm.reset(SynoCoreShare.API_KEY)
        assert not dsm._share

        assert not dsm._system
        assert dsm.system
        assert dsm._system
        assert dsm.reset(SynoCoreSystem.API_KEY)
        assert not dsm._system

        assert not dsm._upgrade
        assert dsm.upgrade
        assert dsm._upgrade
        assert dsm.reset(SynoCoreUpgrade.API_KEY)
        assert not dsm._upgrade

        assert not dsm._utilisation
        assert dsm.utilisation
        assert dsm._utilisation
        assert dsm.reset(SynoCoreUtilization.API_KEY)
        assert not dsm._utilisation

        assert not dsm._download
        assert dsm.download_station
        assert dsm._download
        assert dsm.reset(SynoDownloadStation.API_KEY)
        assert not dsm._download

        assert not dsm._photos
        assert dsm.photos
        assert dsm._photos
        assert dsm.reset(SynoPhotos.API_KEY)
        assert not dsm._photos

        assert not dsm._storage
        assert dsm.storage
        assert dsm._storage
        assert dsm.reset(SynoStorage.API_KEY)
        assert not dsm._storage

        assert not dsm._surveillance
        assert dsm.surveillance_station
        assert dsm._surveillance
        assert dsm.reset(SynoSurveillanceStation.API_KEY)
        assert not dsm._surveillance

    def test_reset_object(self, dsm):
        """Test reset with object."""
        assert not dsm._external_usb
        assert dsm.external_usb
        assert dsm._external_usb
        assert dsm.reset(dsm.external_usb)
        assert not dsm._external_usb

        assert not dsm._security
        assert dsm.security
        assert dsm._security
        assert dsm.reset(dsm.security)
        assert not dsm._security

        assert not dsm._share
        assert dsm.share
        assert dsm._share
        assert dsm.reset(dsm.share)
        assert not dsm._share

        assert not dsm._system
        assert dsm.system
        assert dsm._system
        assert dsm.reset(dsm.system)
        assert not dsm._system

        assert not dsm._upgrade
        assert dsm.upgrade
        assert dsm._upgrade
        assert dsm.reset(dsm.upgrade)
        assert not dsm._upgrade

        assert not dsm._utilisation
        assert dsm.utilisation
        assert dsm._utilisation
        assert dsm.reset(dsm.utilisation)
        assert not dsm._utilisation

        assert not dsm._download
        assert dsm.download_station
        assert dsm._download
        assert dsm.reset(dsm.download_station)
        assert not dsm._download

        assert not dsm._photos
        assert dsm.photos
        assert dsm._photos
        assert dsm.reset(dsm.photos)
        assert not dsm._photos

        assert not dsm._storage
        assert dsm.storage
        assert dsm._storage
        assert dsm.reset(dsm.storage)
        assert not dsm._storage

        assert not dsm._surveillance
        assert dsm.surveillance_station
        assert dsm._surveillance
        assert dsm.reset(dsm.surveillance_station)
        assert not dsm._surveillance

    def test_reset_str_attr_information(self, dsm):
        """Test reset with string information attr (should not be reset)."""
        assert not dsm._information
        assert dsm.information
        assert dsm._information
        assert not dsm.reset("information")
        assert dsm._information

    def test_reset_str_key_information(self, dsm):
        """Test reset with string information API key (should not be reset)."""
        assert not dsm._information
        assert dsm.information
        assert dsm._information
        assert not dsm.reset(SynoDSMInformation.API_KEY)
        assert dsm._information

    def test_reset_object_information(self, dsm):
        """Test reset with information object (should not be reset)."""
        assert not dsm._information
        assert dsm.information
        assert dsm._information
        assert not dsm.reset(dsm.information)
        assert dsm._information

    @pytest.mark.asyncio
    async def test_utilisation(self, dsm):
        """Test utilisation."""
        await dsm.login()
        assert dsm.utilisation
        await dsm.utilisation.update()

    @pytest.mark.asyncio
    async def test_utilisation_cpu(self, dsm):
        """Test utilisation CPU."""
        await dsm.login()
        await dsm.utilisation.update()
        assert dsm.utilisation.cpu
        assert dsm.utilisation.cpu_other_load
        assert dsm.utilisation.cpu_user_load
        assert dsm.utilisation.cpu_system_load
        assert dsm.utilisation.cpu_total_load
        assert dsm.utilisation.cpu_1min_load
        assert dsm.utilisation.cpu_5min_load
        assert dsm.utilisation.cpu_15min_load

    @pytest.mark.asyncio
    async def test_utilisation_error(self, dsm):
        """Test utilisation error."""
        dsm.error = True
        await dsm.login()
        with pytest.raises(SynologyDSMAPIErrorException) as error:
            await dsm.utilisation.update()
        error_value = error.value.args[0]
        assert error_value["api"] == "SYNO.Core.System.Utilization"
        assert error_value["code"] == 1055
        assert error_value["reason"] == "Unknown"
        assert error_value["details"] == {
            "err_key": "",
            "err_line": 883,
            "err_msg": "Transmition failed.",
            "err_session": "",
        }

    @pytest.mark.asyncio
    async def test_utilisation_memory(self, dsm):
        """Test utilisation memory."""
        await dsm.login()
        await dsm.utilisation.update()
        assert dsm.utilisation.memory
        assert dsm.utilisation.memory_real_usage
        assert dsm.utilisation.memory_size()
        assert dsm.utilisation.memory_size(True)
        assert dsm.utilisation.memory_available_swap()
        assert dsm.utilisation.memory_available_swap(True)
        assert dsm.utilisation.memory_cached()
        assert dsm.utilisation.memory_cached(True)
        assert dsm.utilisation.memory_available_real()
        assert dsm.utilisation.memory_available_real(True)
        assert dsm.utilisation.memory_total_real()
        assert dsm.utilisation.memory_total_real(True)
        assert dsm.utilisation.memory_total_swap()
        assert dsm.utilisation.memory_total_swap(True)

    @pytest.mark.asyncio
    async def test_utilisation_network(self, dsm):
        """Test utilisation network."""
        await dsm.login()
        await dsm.utilisation.update()
        assert dsm.utilisation.network
        assert dsm.utilisation.network_up()
        assert dsm.utilisation.network_up(True)
        assert dsm.utilisation.network_down()
        assert dsm.utilisation.network_down(True)



"""DSM Storage data."""

from __future__ import annotations

from typing import TypedDict, cast

from synology_dsm.api import SynoBaseApi
from synology_dsm.helpers import SynoFormatHelper


class SynoStorageDisk(TypedDict, total=False):
    """Synology Storage Disk."""

    id: str
    name: str
    device: str
    firm: str
    diskType: str  # noqa: N815
    smart_status: str
    status: str
    exceed_bad_sector_thr: bool
    below_remain_life_thr: bool
    temp: int
    model: str
    vendor: str
    size_total: int


SynoStoragePoolChild = TypedDict(
    "SynoStoragePoolChild", {"id": str, "size": dict}, total=False
)


class SynoStoragePool(TypedDict, total=False):
    """Synology Storage Pool."""

    disks: list[str]
    pool_child: list[SynoStoragePoolChild]


SynoStorageVolumeSize = TypedDict(
    "SynoStorageVolumeSize",
    {
        "free_inode": str,
        "total": str,
        "total_device": str,
        "total_inode": str,
        "used": str,
    },
)


class SynoStorageVolume(TypedDict, total=False):
    """Synology Storage Volume."""

    id: str
    device_type: str
    size: SynoStorageVolumeSize
    status: str
    fs_type: str


class StorageDataType(TypedDict, total=False):
    """Synology Storage Data type."""

    disks: list[SynoStorageDisk]
    env: dict
    storagePools: list[SynoStoragePool]  # noqa: N815
    volumes: list[SynoStorageVolume]


class SynoStorage(SynoBaseApi[StorageDataType]):
    """Class containing Storage data."""

    API_KEY = "SYNO.Storage.CGI.Storage"

    async def update(self) -> None:
        """Updates storage data."""
        raw_data = await self._dsm.get(self.API_KEY, "load_info")
        if isinstance(raw_data, dict):
            self._data = cast(StorageDataType, raw_data)
            if (data := raw_data.get("data")) is not None:
                self._data = data

    # Root
    @property
    def disks(self) -> list[SynoStorageDisk]:
        """Gets all (internal) disks."""
        return self._data.get("disks", [])

    @property
    def env(self) -> dict | None:
        """Gets storage env."""
        return self._data.get("env")

    @property
    def storage_pools(self) -> list[SynoStoragePool]:
        """Gets all storage pools."""
        return self._data.get("storagePools", [])

    @property
    def volumes(self) -> list[SynoStorageVolume]:
        """Gets all volumes."""
        return self._data.get("volumes", [])

    # Volume
    @property
    def volumes_ids(self) -> list[str]:
        """Returns volumes ids."""
        volumes: list[str] = []
        for volume in self.volumes:
            volumes.append(volume["id"])
        return volumes

    def get_volume(self, volume_id: str) -> SynoStorageVolume | None:
        """Returns a specific volume."""
        for volume in self.volumes:
            if volume["id"] == volume_id:
                return volume
        return None

    def volume_status(self, volume_id: str) -> str | None:
        """Status of the volume (normal, degraded, etc)."""
        if volume := self.get_volume(volume_id):
            return volume.get("status")
        return None

    def volume_device_type(self, volume_id: str) -> str | None:
        """Returns the volume type (RAID1, RAID2, etc)."""
        if volume := self.get_volume(volume_id):
            return volume.get("device_type")
        return None

    def volume_size_total(
        self, volume_id: str, human_readable: bool = False
    ) -> int | str | None:
        """Total size of volume."""
        if (volume := self.get_volume(volume_id)) is None or (
            size := volume.get("size")
        ) is None:
            return None
        return_data = int(size["total"])
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def volume_size_used(
        self, volume_id: str, human_readable: bool = False
    ) -> int | str | None:
        """Total used size in volume."""
        if (volume := self.get_volume(volume_id)) is None or (
            size := volume.get("size")
        ) is None:
            return None
        return_data = int(size["used"])
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def volume_percentage_used(self, volume_id: str) -> float | None:
        """Total used size in percentage for volume."""
        if (volume := self.get_volume(volume_id)) is None or (
            size := volume.get("size")
        ) is None:
            return None
        total = int(size["total"])
        used = int(size["used"])
        return round((float(used) / float(total)) * 100.0, 1)

    def volume_disk_temp_avg(self, volume_id: str) -> float | None:
        """Average temperature of all disks making up the volume."""
        total_temp = 0
        total_disks = 0
        disks = self._get_disks_for_volume(volume_id)
        for disk in disks:
            if disk_temp := self.disk_temp(disk["id"]):
                total_disks += 1
                total_temp += disk_temp

        if total_temp > 0 and total_disks > 0:
            return round(total_temp / total_disks, 0)
        return None

    def volume_disk_temp_max(self, volume_id: str) -> int | None:
        """Maximum temperature of all disks making up the volume."""
        disks = self._get_disks_for_volume(volume_id)
        if not disks:
            return None

        disk_temps: list[int] = [0]
        for disk in disks:
            if disk_temp := self.disk_temp(disk["id"]):
                disk_temps.append(disk_temp)
        return max(disk_temps)

    # Disk
    @property
    def disks_ids(self) -> list[str]:
        """Returns (internal) disks ids."""
        disks: list[str] = []
        for disk in self.disks:
            disks.append(disk["id"])
        return disks

    def get_disk(self, disk_id: str) -> SynoStorageDisk | None:
        """Returns a specific disk."""
        for disk in self.disks:
            if disk["id"] == disk_id:
                return disk
        return None

    def _get_disks_for_volume(self, volume_id: str) -> list[SynoStorageDisk]:
        """Returns a list of disk for a specific volume."""
        disks: list[SynoStorageDisk] = []
        for pool in self.storage_pools:
            if pool.get("deploy_path") == volume_id:
                # RAID disk redundancy
                for disk_id in pool["disks"]:
                    if disk := self.get_disk(disk_id):
                        disks.append(disk)

            if pool.get("pool_child"):
                # SHR disk redundancy
                for pool_child in pool["pool_child"]:
                    if pool_child["id"] != volume_id:
                        continue
                    for disk_id in pool["disks"]:
                        if disk := self.get_disk(disk_id):
                            disks.append(disk)

        return disks

    def disk_name(self, disk_id: str) -> str | None:
        """The name of this disk."""
        if disk := self.get_disk(disk_id):
            return disk.get("name")
        return None

    def disk_device(self, disk_id: str) -> str | None:
        """The mount point of this disk."""
        if disk := self.get_disk(disk_id):
            return disk.get("device")
        return None

    def disk_smart_status(self, disk_id: str) -> str | None:
        """Status of disk according to S.M.A.R.T)."""
        if disk := self.get_disk(disk_id):
            return disk.get("smart_status")
        return None

    def disk_status(self, disk_id: str) -> str | None:
        """Status of disk."""
        if disk := self.get_disk(disk_id):
            return disk.get("status")
        return None

    def disk_exceed_bad_sector_thr(self, disk_id: str) -> bool | None:
        """Checks if disk has exceeded maximum bad sector threshold."""
        if disk := self.get_disk(disk_id):
            return disk.get("exceed_bad_sector_thr")
        return None

    def disk_below_remain_life_thr(self, disk_id: str) -> bool | None:
        """Checks if disk has fallen below minimum life threshold."""
        if disk := self.get_disk(disk_id):
            return disk.get("below_remain_life_thr")
        return None

    def disk_temp(self, disk_id: str) -> int | None:
        """Returns the temperature of the disk."""
        if disk := self.get_disk(disk_id):
            return disk.get("temp")
        return None



"""SurveillanceStation camera."""

from __future__ import annotations

from typing import TypedDict

from .const import MOTION_DETECTION_DISABLED, RECORDING_STATUS

SynoCameraData = TypedDict(
    "SynoCameraData",
    {
        "enabled": bool,
        "fps": int,
        "id": int,
        "MDParam": dict,
        "model": str,
        "name": str,
        "recStatus": int,
        "resolution": str,
    },
    total=False,
)

SynoCameraLifeViewData = TypedDict(
    "SynoCameraLifeViewData",
    {
        "mjpegHttpPath": str,
        "multicstPath": str,
        "mxpegHttpPath": str,
        "rtspOverHttpPath": str,
        "rtspPath": str,
    },
    total=False,
)


class SynoCamera:
    """An representation of a Synology SurveillanceStation camera."""

    def __init__(
        self, data: SynoCameraData, live_view_data: SynoCameraLifeViewData | None = None
    ) -> None:
        """Initialize a Surveillance Station camera."""
        self._data: SynoCameraData = data
        self.live_view = SynoCameraLiveView(live_view_data)
        self._motion_detection_enabled: bool | None = None

    def update(self, data: SynoCameraData) -> None:
        """Update the camera."""
        self._data = data

    def update_motion_detection(self, data: SynoCameraData) -> None:
        """Update the camera motion detection."""
        self._motion_detection_enabled = (
            MOTION_DETECTION_DISABLED != data["MDParam"]["source"]
        )

    @property
    def id(self) -> int:
        """Return id of the camera."""
        return self._data["id"]

    @property
    def name(self) -> str:
        """Return name of the camera."""
        return self._data["name"]

    @property
    def model(self) -> str:
        """Return model of the camera."""
        return self._data["model"]

    @property
    def resolution(self) -> str:
        """Return resolution of the camera."""
        return self._data["resolution"]

    @property
    def fps(self) -> int:
        """Return FPS of the camera."""
        return self._data["fps"]

    @property
    def is_enabled(self) -> bool:
        """Return true if camera is enabled."""
        return self._data["enabled"]

    @property
    def is_motion_detection_enabled(self) -> bool | None:
        """Return true if motion detection is enabled."""
        return self._motion_detection_enabled

    @property
    def is_recording(self) -> bool:
        """Return true if camera is recording."""
        return self._data["recStatus"] in RECORDING_STATUS


class SynoCameraLiveView:
    """An representation of a Synology SurveillanceStation camera live view."""

    def __init__(self, data: SynoCameraLifeViewData | None):
        """Initialize a Surveillance Station camera live view."""
        if data is not None:
            self._data = data
        else:
            self._data = {}

    def update(self, data: SynoCameraLifeViewData) -> None:
        """Update the camera live view."""
        self._data = data

    @property
    def mjpeg_http(self) -> str | None:
        """Return the mjpeg stream (over http) path of the camera."""
        return self._data.get("mjpegHttpPath")

    @property
    def multicast(self) -> str | None:
        """Return the multi-cast path of the camera."""
        return self._data.get("multicstPath")

    @property
    def mxpeg_http(self) -> str | None:
        """Return the mxpeg stream path of the camera."""
        return self._data.get("mxpegHttpPath")

    @property
    def rtsp_http(self) -> str | None:
        """Return the RTSP stream (over http) path of the camera."""
        return self._data.get("rtspOverHttpPath")

    @property
    def rtsp(self) -> str | None:
        """Return the RTSP stream path of the camera."""
        return self._data.get("rtspPath")



"""DSM 5 SYNO.Core.System.Utilization data."""

DSM_5_CORE_UTILIZATION = {
    "data": {
        "cpu": {
            "15min_load": 53,
            "1min_load": 57,
            "5min_load": 56,
            "device": "System",
            "other_load": 63,
            "system_load": 10,
            "user_load": 27,
        },
        "disk": {
            "disk": [
                {
                    "device": "sda",
                    "display_name": "Disk 1",
                    "read_access": 21,
                    "read_byte": 645529,
                    "type": "internal",
                    "utilization": 46,
                    "write_access": 4,
                    "write_byte": 86220,
                },
                {
                    "device": "sdb",
                    "display_name": "Disk 2",
                    "read_access": 23,
                    "read_byte": 711338,
                    "type": "internal",
                    "utilization": 33,
                    "write_access": 4,
                    "write_byte": 95641,
                },
                {
                    "device": "sdc",
                    "display_name": "Disk 3",
                    "read_access": 21,
                    "read_byte": 786841,
                    "type": "internal",
                    "utilization": 31,
                    "write_access": 5,
                    "write_byte": 99874,
                },
                {
                    "device": "sdd",
                    "display_name": "Disk 4",
                    "read_access": 21,
                    "read_byte": 729907,
                    "type": "internal",
                    "utilization": 32,
                    "write_access": 4,
                    "write_byte": 76663,
                },
                {
                    "device": "sdq",
                    "display_name": "USB Disk 1",
                    "read_access": 0,
                    "read_byte": 0,
                    "type": "usb",
                    "utilization": 0,
                    "write_access": 0,
                    "write_byte": 0,
                },
            ],
            "total": {
                "device": "total",
                "read_access": 86,
                "read_byte": 2873615,
                "utilization": 28,
                "write_access": 17,
                "write_byte": 358398,
            },
        },
        "memory": {
            "avail_real": 8188,
            "avail_swap": 1933436,
            "buffer": 3700,
            "cached": 25636,
            "device": "Memory",
            "memory_size": 131072,
            "real_usage": 68,
            "si_disk": 5,
            "so_disk": 3,
            "swap_usage": 7,
            "total_real": 118464,
            "total_swap": 2097080,
        },
        "network": [
            {"device": "total", "rx": 1680, "tx": 553},
            {"device": "eth0", "rx": 1680, "tx": 553},
        ],
        "space": {
            "lun": [],
            "total": {
                "device": "total",
                "read_access": 261,
                "read_byte": 1069875,
                "utilization": 100,
                "write_access": 51,
                "write_byte": 208896,
            },
            "volume": [
                {
                    "device": "md2",
                    "display_name": "volume1",
                    "read_access": 261,
                    "read_byte": 1069875,
                    "utilization": 100,
                    "write_access": 51,
                    "write_byte": 208896,
                }
            ],
        },
        "time": 1586592505,
    },
    "success": True,
}
